﻿using System;
using System.Collections.Generic;
using System.Linq;
using Composite.Data;
using Composite.Data.Plugins.DataProvider;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using Composite.Core.Threading;

namespace C1NorthwindIntegration
{
    [ConfigurationElementType(typeof(NonConfigurableDataProvider))]
    public class NorthwindDataProvider : IDataProvider
    {
        /// <summary>
        /// This is access from the ProductBase and SupplierBase, so they are able to create DataSourceId's 
        /// </summary>
        internal static DataProviderContext DataProviderContextHolder;


        public NorthwindDataProvider()
        {
        }


        /// <summary>
        /// This is set by C1 when the data provider is created.
        /// 
        /// DataProviderContext is used for creating DataSourceId's. The data provider should give the id (like key in sql) for a item to create a DataSourceId.
        /// This id is contained in a IDataId implementation. When asked the data provider should be able to deliver a data item given this
        /// IDataId implementation. Usally the IDataId contains the sql key value for the given item. 
        /// In this example the IDataId implementaion contains an integer since the Northwind DB uses ints as keys for data entires.
        /// 
        /// DataSourceId are unique data identifiers for data items in C1. DataSourceId covers the id for the data item (like a key in sql). 
        /// It also contains the provider the data item comes from and which locale/language the data item belongs to.
        /// </summary>
        public DataProviderContext Context
        {
            set 
            {
                // This will not work if this provider is used more than once in a C1 solution
                // If more than one of this provider is needed, a more complex solution should be developed.
                DataProviderContextHolder = value;
            }
        }



        /// <summary>
        /// Returns the C1 data interfaces that this provider supports
        /// </summary>
        /// <returns></returns>
        public IEnumerable<Type> GetSupportedInterfaces()
        {
            return new Type[] { typeof(ISupplier), typeof(IProduct) };
        }



        private NorthwindEntities GetEntityContext()
        {
            NorthwindEntities context = new NorthwindEntities();

            ThreadDataManager.GetCurrentNotNull().OnDispose += () =>
            {
                context.Dispose();
            };

            return context;
        }


        /// <summary>
        /// Returns an IQueryable given the C1 data interface type (generic).
        /// Here we just return the Table instance from the data entities instance.
        /// We happily use covariance from .NET 4.0, woohoo :)
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        public IQueryable<T> GetData<T>() where T : class, Composite.Data.IData
        {
            if (typeof(T) == typeof(IProduct))
            {
                IQueryable<IProduct> products = GetEntityContext().Products;

                return (IQueryable<T>)products;
            }
            else if (typeof(T) == typeof(ISupplier))
            {
                IQueryable<ISupplier> supplier = GetEntityContext().Suppliers;

                return (IQueryable<T>)supplier;
            }

            throw new NotImplementedException();
        }



        /// <summary>
        /// This method should return a data item given a IDataId implementation instance.
        /// In this example we use a simple IDataId implementation that contains one integer.
        /// This integer is the key value of a item in the Northwind DB.
        /// We use linq (Where) to get the right item and return it.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="dataId"></param>
        /// <returns></returns>
        public T GetData<T>(IDataId dataId) where T : class, Composite.Data.IData
        {
            DataId intDataId = (DataId)dataId;

            if (typeof(T) == typeof(IProduct))
            {
                IProduct product = GetEntityContext().Products.Where(f => f.ProductID == intDataId.Id).Single();
                return (T)product;
            }
            else if (typeof(T) == typeof(ISupplier))
            {
                ISupplier supplier = GetEntityContext().Suppliers.Where(f => f.SupplierID == intDataId.Id).Single();
                return (T)supplier;
            }

            throw new NotImplementedException();
        }
    }
}
